# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots



class Cameras(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
		# tree = self.sb.cameras_lower_submenu.tree000
		# tree.expandOnHover = True
		# tree.convert(tree.getTopLevelItems(), 'QLabel') #construct the tree using the existing contents.

		
